/* STARTUP CODE FOR ADuC7026 */

/*---------------------------*/
/* DEFINITIONS */
/*---------------------------*/
	/* Operating Modes (Refer to ARM7 data sheet) */
		.equ	User_Mode,	0x10
		.equ	FIQ_Mode,	0x11
		.equ	IRQ_Mode,	0x12
		.equ	Sup_Mode,	0x13
		.equ	Abort_Mode,	0x17
		.equ	Undef_Mode,	0x1B
		.equ	Sys_Mode,	0x1F
	/* IRQ/FIQ Enable Masks (Disabled in startup. May be re-enabled in Main if needed) */
		.equ	NO_IRQ,		0x80
		.equ	NO_FIQ,		0x40
		
/*---------------------------*/
/*Exception/Interrupt Vectors*/
/*---------------------------*/
	/*Must be linked to Address 0x0. If memory remap is performed, exception table needs to be set up */
	/*in RAM FIRST! (Refer to ARM7 data sheet) */
	/*Vector table is dummy table. Only the reset (first instruction) does */
	/*something sensible, the others are infinite loops resulting in stalling the MCU. */
	/*This implementation is possible as such exceptions cannot occur at this time on a functional MCU! */

	.section "flash_exept_vecs"
		reset:		B	_start			/* startup_routine performed after each Reset */
		undef:		B	undef
		swi:		B	swi
		pref_abort:	B	pref_abort
		data_abort:	B	data_abort
					NOP
		irq:		B	irq
		fiq:		B	fiq
	
/*--------------------------*/
/*TEXT SECTION */
/*--------------------------*/
	/*From here ojn something happens*/
	
	.text
	.arm	/*select 32-Bit ARM instruction set */
	
	.global		_start
	/*.func		_start								/* only needed for Debug. Needs .endfunc at end */
	
	
	_start:											/* Executet after every reste */
	/*First Call the low level initialisation implemented in C */
		LDR		LR, =end_C_start					/* set return address MOV LR,PC not possible due to potential mem-remap */
		LDR		SP, =_STACK_TOP_					/* !!temporary!! stack setup */
		LDR		R0, =_start							/* Pass start (reste) address as first argument */
		B		low_level_init						/* call to C-function void low_level_init(void) */
	end_C_start:
	
				/********************************************************/
				/* Eventual Memory Remap is performed in low-level-init */
				/* Rest of Code executes from Linked location */
				/* Secondary Exception-Vector-Table is in place already */
				/********************************************************/
	
	/* relocate .fastcode section to RAM (critical code will be executed faster from RAM) */
	copyfcode:
		LDR		R0, =_FLASH_FCODE_START_
		LDR		R1, =_RAM_FCODE_START_
		LDR		R2, =_FCODE_LENGTH_
		CMP		R2, #0							/* Handle 0-Size Fastcode-Section */
		BEQ		endcopyfcode
	fcodeLoop:
		LDRLO	R3, [R0], #4					/* Load from FLASH + increase R0 */
		STRLO	R3, [R1], #4					/* Store in RAM + increase R1 */
		subs	R2,R2, #4						/* check if finished */
		BNE		fcodeLoop
	endcopyfcode:
	
	/*relocate .data Section to RAM	*/
	copyData:
		LDR		R0, =_FLASH_DATA_START_
		LDR		R1, =_RAM_DATA_START_
		LDR		R2, =_DATA_LENGTH_
		CMP		R2, #0							/* Handle 0-Size Data Section */
		BEQ		endCopyData
	dataLoop:
		LDRLO	R3, [R0], #4					/* Load from FLASH + increase R0*/
		STRLO	R3, [R1], #4					/* Store in RAM + increase R1*/
		subs	R2,R2, #4						/* check if finished */
		BNE		dataLoop
	endCopyData:
	
	/*clear the .bb Section in RAM*/
	clearBss:
		LDR		R1, =_BSS_START_
		LDR		R2, =_BSS_END_
		CMP		R1, R2							/* Handle 0-Size Bss Section */
		BEQ		endClearBss
		MOV		R0, #0
	bssLoop:
		CMP		R1,R2							/* check if finished */
		STRLO	R0, [R1], #4					/* store 0x0 (C-Standard) in RAM + increase R1 */
		BLO		bssLoop
	endClearBss:
	
	/*setup stack for all modes*/
	stackSetup:										/* Enter Modes, disable interrupts, set stack-pointer */
	
		/* Fll stack with all '1's */
	stackFill:
		LDR		R1, =_STACK_START_
		LDR		R2, =_STACK_TOP_
		MOV		R0, #0xFFFFFFFF
	stackfillLoop:
		CMP		R1, R2
		STRLO	R0, [R1], #4
		BLO		stackfillLoop
	endStackFill:
		
		/* FIQ_Mode */
		MSR		CPSR_c, #(FIQ_Mode|NO_IRQ|NO_FIQ)	/* CPSR is current-program-status-register */
		LDR		SP, =_FIQ_STACK_TOP_				/* _c -> only last 8 Bits are addressed */
		/* IRQ_Mode */
		MSR		CPSR_c, #(IRQ_Mode|NO_IRQ|NO_FIQ)
		LDR		SP, =_IRQ_STACK_TOP_
		/* Sup_Mode */
		MSR		CPSR_c, #(Sup_Mode|NO_IRQ|NO_FIQ)
		LDR		SP, =_SUP_STACK_TOP_
		/* Abort_Mode */
		MSR		CPSR_c, #(Abort_Mode|NO_IRQ|NO_FIQ)
		LDR		SP, =_ABORT_STACK_TOP_
		/*Undef_Mode */
		MSR		CPSR_c, #(Undef_Mode|NO_IRQ|NO_FIQ)
		LDR		SP, =_UNDEF_STACK_TOP_
		/* Sys_Mode */
			/*not set*/
		/* User_Mode */
		MSR		CPSR_c, #(User_Mode|NO_IRQ|NO_FIQ)
		LDR		SP, =_USER_STACK_TOP_
		
/*--------------------------*/
/*BRANCH TO MAIN*/
/*--------------------------*/

	/*Branch to "int main(void)"*/
	start_C_Code:
		LDR		R0, =mymain
		MOV		LR, PC									/* Return address: Main should never return */
		BX		R0										/* X: T-Bit checked to determine Thumb or ARM */
		
	/*This Part should never be reached. If it does -> infinite Loop*/
	stop:
		B		stop
		
/*--------------------------*/
/*END (Closing functions etc.)*/
/*--------------------------*/

		/*.size _startup_routine, . - _startup_routine	/* Only needed for Debug */
		/*.endfunc										/* only if .func is enabled in beginning */
		
		.end
		